<?php

declare(strict_types=1);

namespace Drupal\field_data;

use Drupal\Core\Breadcrumb\Breadcrumb;
use Drupal\Core\Breadcrumb\BreadcrumbBuilderInterface;
use Drupal\Core\Link;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a breadcrumb builder for the 'Field data' report.
 */
final class FieldDataBreadcrumbBuilder implements BreadcrumbBuilderInterface {

  use StringTranslationTrait;

  /**
   * Constructs a FieldDataBreadcrumbBuilder object.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   */
  public function __construct(
    protected RequestStack $requestStack,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function applies(RouteMatchInterface $route_match): bool {
    return $route_match->getRouteName() === 'field_data.report';
  }

  /**
   * {@inheritdoc}
   */
  public function build(RouteMatchInterface $route_match): Breadcrumb {
    $breadcrumb = new Breadcrumb();

    // This breadcrumb builder is based on query arguments.
    $breadcrumb->addCacheContexts([
      'url.query_args:entity_type',
      'url.query_args:bundle',
      'url.query_args:field',
    ]);

    $breadcrumb->addLink(Link::createFromRoute($this->t('Home'), '<front>'));
    $breadcrumb->addLink(Link::createFromRoute($this->t('Administration'), 'system.admin'));
    $breadcrumb->addLink(Link::createFromRoute($this->t('Reports'), 'system.admin_reports'));
    $breadcrumb->addLink(Link::createFromRoute($this->t('Field list'), 'entity.field_storage_config.collection'));

    $request = $this->requestStack->getCurrentRequest();
    $properties = array_filter([
      'entity_type' => $request->query->get('entity_type'),
      'bundle' => $request->query->get('bundle'),
      'field' => $request->query->get('field'),
    ]);

    if (empty($properties)) {
      return $breadcrumb;
    }

    $breadcrumb->addLink(Link::createFromRoute($this->t('Field data'), 'field_data.report'));

    // Remove the current page.
    array_pop($properties);

    $query = [];
    foreach ($properties as $key => $value) {
      $query[$key] = $value;
      $breadcrumb->addLink(Link::createFromRoute($value, 'field_data.report', [], ['query' => $query]));
    }

    return $breadcrumb;
  }

}
